﻿#include "sysusermgtv2.h"
#include <QHBoxLayout>
#include <QToolButton>
#include <QToolBar>
#include <QPushButton>
#include <topcore/topcore.h>
#include <twidget/taccordion.h>
#include <twidget/tsplitter.h>
#include <tbaseutil/ttheme.h>
#include <twidget/tuiloader.h>
#include <twidget/tsearchentry.h>
#include <tbaseutil/tresource.h>
#include <qframe.h>
#include <twidget/ttableview.h>
#include <twidget/tpagetool.h>
#include <topcore/topclassabs.h>
#include <topcore/topclasshelper.h>
#include <tbaseutil/tdataresponse.h>
#include <toputil/t.h>
#include <topcore/topclasssqlthread.h>
#include <tbaseutil/tdataparse.h>
#include "sysuserv2.h"
#include "sysusermgtthreadv2.h"

SysUserMgtV2::SysUserMgtV2(const QString &iModuleNameStr,
                           const QVariantMap &iUrlPars,
                           QWidget *iParent)
    : TopClassAbs(iParent)
{
    this->appendLanguage("sys-user");
    this->setLicenseKey("sys_common");
    this->initModule(iModuleNameStr, iUrlPars);
    QVariant prodCfg = config("product_category");

    if (prodCfg.type() == QVariant::List || prodCfg.type() == QVariant::StringList) {
        for (QVariant row : prodCfg.toList()){
            mProductCategoryLst.append(row.toString());
        }
    } else if (!prodCfg.toString().isEmpty()) {
        mProductCategoryLst.append(prodCfg.toString());
    }
    if (mProductCategoryLst.isEmpty()) {
        mProductCategoryLst.append(APP->productCategory());
    }

    /* 现在有一个完整的空白界面
     * 开始分割页面，页面分为3部分：
     */
    mBodySplitter = new TSplitter(this);
    this->setCentralWidget(mBodySplitter);

    // 第一部分是左边导航栏
    // 设置导航栏
    initNaviView();

    // 第二部分是中间的表格
    // 设置表格，给表格以表头、搜索、翻页功能
    initTableView();
    // 右键弹窗
    if (QMenu *table_popup = qobject_cast<QMenu *>(uim()->getWidget("TABLEVIEW_POPUP"))) {
        mTableView->setContextMenu(table_popup);
    }

    // 第三部分是最右边的用户详情页
    QString detailModuleName = config("detail_module_name").toString();
    if (detailModuleName.isEmpty()) {
        detailModuleName = "sys-user-v2";
    }
    mDetailView = new SysUserV2(detailModuleName, config(), this);
    connect(mDetailView, SIGNAL(dataSaved(QVariant)), this, SLOT(onDetailSaved(QVariant)));
    connect(mDetailView, SIGNAL(windowModifyChanged(bool)), SLOT(onDetailChanged()));
    mBodySplitter->addWidget(mDetailView);

    mDetailView->resize(TTHEME_DP(config("detail.perfect_size.width", mDetailView->perfectSize().width()).toInt()), mDetailView->height());
    mBodySplitter->setStretchFactor(0, 1);

    this->restoreSizeState();
    this->restoreObjectState(mTableView);

    refreshActionState();
    QTimer::singleShot(0, this, SLOT(refresh()));
}

SysUserMgtV2::~SysUserMgtV2()
{
    this->saveSizeState();
    this->saveObjectState(mTableView);
}

// 设置导航栏开始
void SysUserMgtV2::initNaviView()
{
    TAccordion *accordion = new TAccordion(this);
    // 设置框体布局
    accordion->setProperty("SS_BG","NAVI");
    accordion->setMinimumWidth(TTHEME_DP(config("navi.min_width", 150).toInt()));
    accordion->setMaximumWidth(TTHEME_DP(config("navi.min_width", 300).toInt()));
    accordion->setAccordionLocation(TAccordion::AccordionLocation::Left);
    accordion->setIsExpanded(true);
    QGridLayout *naviLayout = new QGridLayout(accordion);
    naviLayout->setMargin(0);
    naviLayout->setSpacing(0);
    // 设置框体上的部件
    // 首先是最上面的导航栏三个字和灰色三角图标
    QLabel *naviLabel = new QLabel(ttr("Navigation"));
    naviLabel->setProperty("SS_TYPE","SUBHEADER");
    naviLayout->addWidget(naviLabel);
    mBodySplitter->insertWidget(0,accordion);
    accordion->expandButton()->setFixedHeight(TTHEME_DP(40));
    accordion->expandButton()->setToolTip(ttr("Show Navigation"));
    accordion->collapseButton()->setToolTip(ttr("Hide Navigation"));
    // 然后在导航栏下设置登录名等有输入功能的控件
    // 使用TUiLoader通过JavaScript生成动态界面
    mNaviView = new TUiLoader(this);
    TUiLoader *filterUi = mNaviView;
    filterUi->setSelf(this);
    /* 现在的filterUi是空的，没有脚本解析功能，
    * 因为它指向了setScriptEngine(APP->scriptEngine())
    * 所以才有了解析脚本的功能
    */
    filterUi->setScriptEngine(APP->scriptEngine());
    filterUi->setUiStr(ui("navi").toString());
    // 开始把Ui界面放入布局里面
    QFrame *firstWgt = new QFrame(this);
    QVBoxLayout *firstLayout= new QVBoxLayout(firstWgt);
    firstLayout->insertWidget(0,mNaviView,0);
    naviLayout->addWidget(firstWgt);
    // 把按钮放入布局里
    // 取消按钮
    QFrame *secendWgt = new QFrame(this);
    QHBoxLayout *btnLayout = new QHBoxLayout(secendWgt);
    btnLayout->setMargin(10);
    btnLayout->setSpacing(10);
    QPushButton *resetBtn = new QPushButton(this);
    resetBtn->setText(ttr("Cancel"));
    resetBtn->setIcon(TRES->icon("rotate-right"));
    connect(resetBtn,SIGNAL(clicked(bool)),this,SLOT(onNaviReset()));
    btnLayout->addWidget(resetBtn);
    // 确定按钮
    QPushButton *filterBtn = new QPushButton(this);
    filterBtn->setText(ttr("Ok"));
    filterBtn->setIcon(TRES->icon("filter"));
    connect(filterBtn,SIGNAL(clicked(bool)),this,SLOT(refresh()));
    btnLayout->addWidget(filterBtn);
    naviLayout->addWidget(secendWgt);
    QWidget *stretcher = new QWidget(this);
    //Expanding的作用：当页面有空白的时候，它会尽可能的把空白挤掉
    stretcher->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    naviLayout->addWidget(stretcher);
}
// 设置导航栏结束

// 设置表格开始
void SysUserMgtV2::initTableView()
{
    mBodyWidget = new QWidget(this);
    QVBoxLayout *vboxlayout = new QVBoxLayout(mBodyWidget);
    vboxlayout->setMargin(0);
    vboxlayout->setSpacing(0);
    // 新建表格
    mTableView = new TTableView(this);
    // 设置表格
    // 给表格添加顶头行,现在的顶头行里还没有文字
    // QVariantList headerItem = mTableConf->horizontalHeaders;
    mTableConf = new TopClassTableConf;
    //把view里面的键值对给到mTableConf
    TopClassHelper::parseTableConf0(this,"view",mTableConf);
    // 给表格头添加文字
    QVariantList header = mTableConf->horizontalHeaders;
    header.prepend(QVariant());
    // 将表头添加进表格中
    mTableView->setHeaderItem(header);
    // 给表设置键和主键
    mTableView->setDataKeyList(mTableConf->dataKeys);
    mTableView->setPrimaryKey(mTableConf->primaryKey);
    // 创建连接：当用户点击顶头栏图标时，切换数据的排列顺序
    QHeaderView *horizontalHeader = mTableView->horizontalHeader();
    connect(horizontalHeader,SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)),
            this,SLOT(refresh()));
    // 设计一个搜索框，此时还没有搜索功能
    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("MAIN_TOOLBAR"))){
        toolbar->setWindowTitle(ttr("ToolBar"));
        vboxlayout->addWidget(toolbar,0);
    }
    mSearchEntry = qobject_cast<TSearchEntry *>(uim()->getWidget("MAIN_TOOLBAR/SEARCH_ENTRY"));
    // 实现在搜索框中的搜索功能
    if (mSearchEntry != nullptr) {
        mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mTableView->headerItem()));
        mSearchEntry->setPlaceholderText(ttr("Search %1"));
        connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refresh()));
        this->setFocusProxy(mSearchEntry);
    }
    // 将表格放入垂直布局
    vboxlayout->addWidget(mTableView);
    // 将中间表格放入分隔中,以显示表格，此时，表头已经显示出来
    mBodySplitter->addWidget(mBodyWidget);
    //在表格上添加右键->弹出窗口的功能
    // 在表格下面添加翻页工具
    if (QToolBar *toolbar = qobject_cast<QToolBar*>(uim()->getWidget("BOTTOM_TOOLBAR"))) {
        vboxlayout->addWidget(toolbar, 0);
        mPageTool = qobject_cast<TPageTool *>(uim()->getWidget("BOTTOM_TOOLBAR/PAGE_TOOL"));
    }
    // 翻页功能
    if (mPageTool != nullptr) {
        mPageTool->setPageSizeVisible(true);
        // 当用户在翻页工具发出换页信号时，当前页面改变
        connect(mPageTool, SIGNAL(pageChanged(int,int)), this, SLOT(onPageChanged()));
    }
    // 切换功能
    connect(mTableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            this, SLOT(onSelectionChanged()));
}
// 设置表格结束

// ”增“功能开始
void SysUserMgtV2::newItem() {
    // 首先是用户信息栏进入默认信息
    mDetailView->create();
    // 获取用户输入的数据->要保存数据必须先获取数据，所以获取数据这一步在保存里已经实现
    // 最后是刷新当前模块控件的状态
    mDetailView->refreshActionState();
}
// ”增“功能结束

// 删除功能开始
void SysUserMgtV2::deleteItem(const QVariantList &iIdLst)
{
    // 不能删除当前登录的用户,不能删除ADMIN用户
    if (selectedNames().contains(APP->userName())) {
        alertError(ttr("Current login user can not be deleted!"));
        return;
    } else if(selectedNames().contains("admin",Qt::CaseInsensitive)) {
        alertError(ttr("Admin can not be deleted!"));
        return;
    } else {
        QVariant data = doThreadWork(new SysUserMgtThreadV2(this), "DELETE_USER", iIdLst);
        TDataResponse dataRes(data.toMap());
        if (dataRes.hasError()) {
            alertError(ttr("Delete data failed!"), dataRes.errText());
        } else {
            refresh(true);
            alertOk(ttr("Data deleted"));
        }
    }
}
// 删除功能结束

// 改功能开始
void SysUserMgtV2::changeItem()
{
    // 先选择一条信息
    mSelectedLst = mTableView->selectedPrimaryKeys();
    // 删除原来的数据
    this->deleteItem(mSelectedLst);
    // 然后新建一条信息
    this->newItem();
}
// 改功能结束

// 查询功能开始
void SysUserMgtV2::searchFunc()
{
    if (mSearchEntry != nullptr) {
        // 设置选项列表->出现点击放大镜显示的列表 TDataParse::headerItem2searchList()函数可以去头标题的前三个，放到放大镜选项下
        mSearchEntry->setOptionList(TDataParse::headerItem2searchList(mTableView->headerItem()));
        // 在搜索框里放搜索两个字
        mSearchEntry->setPlaceholderText(ttr("Search %1"));
        // 当用户回车就会调用搜索框的查询函数search()，然后在当前页面刷新显示搜索内容
        connect(mSearchEntry, SIGNAL(search(QString,QVariant)), this, SLOT(refresh()));
        //聚焦功能：当鼠标点击框框之后，会选中框框的功能
        this->setFocusProxy(mSearchEntry);
    }
}
// 查询功能结束

// 刷新功能开始
void SysUserMgtV2::refresh(bool iResetPageBol)
{
    t::loading(this);
    // 获取一个SQL查询器
    TSqlSelectorV2 selector = getSqlSelector(iResetPageBol);
    // doThreadWork返回查询数据库的结果，并将查询结果返回给data
    QVariant data = doThreadWork(new TopClassSqlThread(this), TOPSQLTHREAD_SELECT_ARRAYMAP, QVariant::fromValue(selector));
    unloading();
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Load data failed!"));
    } else {
        // 将数据展示在中间的表格上
        fillTableData(dataRes);
        alertOk(ttr("Data loaded"));
    }
    mDetailView->setUid(0);
}
// 刷新功能结束

// 中间页面是否可以被修改开始：如果详情页正在输入数据，那么中间页面不可修改
bool SysUserMgtV2::canModify()
{
    return !(mDetailView->isDataModified());
}
// 中间页面是否可以被修改结束

// 如果详情页在更改数据，那么中间页蒙上膜 开始
void SysUserMgtV2::onDetailChanged()
{
    // 正在详情页输入数据->数据正在被修改
    bool isModified = mDetailView->isDataModified();
    // 所以不能操作中间的表格页面
    mTableView->setEnabled(!isModified);
    // 如果中间的表格不能被操作，那就给它蒙上一层膜
    if (isModified) {
        this->showMaskFrame(mBodyWidget);
    } else {
        if (mBodySplitter->sizes().value(0) < 20) {
            mBodySplitter->setWidgetSize(0, mBodySplitter->getCacheSize(0), true);
        }
        this->hideMaskFrame();
    }
    refreshActionState();
    mDetailView->refreshActionState();
}
//如果详情页在更改数据，那么中间页蒙上膜 结束

// 换页之后刷新页面开始
//就是刷新功能，因为参数是false，所以刷新的从当前页面开始刷新，而不是刷新全部页面的数据
void SysUserMgtV2::onPageChanged()
{
    refresh(false);
}
// 换页之后刷新页面结束

// 在中间页填数据开始
void SysUserMgtV2::fillTableData(const TDataResponse &iDataRes)
{
    if (mPageTool != nullptr) {
        mPageTool->setRowCount(iDataRes.dataCount(), true);
    }
    // 将数据库中返回的值转换为data的样式，再转换为List()的样式
    QVariantList dataLst = iDataRes.data().toList();
    TopClassHelper::formatTableData(this,mTableConf,dataLst);
    mTableView->loadData(dataLst);
}
// 在中间页填数据结束

// 选择一个用户名开始
QStringList SysUserMgtV2::selectedNames()
{
    QStringList retLst;
    for (QVariant item : mTableView->selectedRowDataMaps()) {
        retLst << item.toMap().value("username").toString();
    }
    return retLst;
}
// 选择一个用户名结束

// 一条查询语句
TSqlSelectorV2 SysUserMgtV2::getSqlSelector(bool iResetPageBol)
{
    // 一个查询器
    TSqlSelectorV2 selector;
    // dbSql查sys_user和pub_contacts左连接而成的表里的id值相等的一些数据
    // 如果能查到数据
    if (!mTableConf->dbSql.isEmpty()) {
        // 查询器就会设置表格的名字为表名_TEMP_TABLE_
        selector.setTable(QString("(%1) _TEMP_TABLE_").arg(mTableConf->dbSql));
    } else {
        selector.setTable(mTableConf->dbTableName);
    }
    // 返回表的行数
    selector.setReturnRowCount(true);
    //设置有哪些字段
    selector.setField(mTableConf->queryFields);
    //设置字段格式
    selector.setFieldFormat("product_category", "array");
    // 操纵搜索页面                              搜索入口    底部工具栏   重置工具栏       查询器
    TopClassHelper::handleSearchPageOnQuery(mSearchEntry, mPageTool, iResetPageBol, &selector);
    //如果导航栏不存在  并且
    if (mNaviView != nullptr && !uiloaderWhere().isEmpty()) {
        selector.addWhere(uiloaderWhere());
    }
    //查询器设置的顺序是按照id 来查 逆序
    selector.setOrder("id", Qt::DescendingOrder);
    // 又在表格中创建了一个头行
    QHeaderView *headerView = mTableView->horizontalHeader();
    if (headerView != nullptr) {
        QString orderField = mTableView->columnName(headerView->sortIndicatorSection());
        if (!orderField.isEmpty() && orderField != "remark") {
            TopClassHelper::handleOrderOnQuery(mTableView, mTableConf, &selector);
        }
    }
    // 当product_category包含 * 时,显示所有
    //Compsite:合成的
    TSqlWhereCompsiteV2 whereCompsite;
    if (!mProductCategoryLst.contains("*")) {
        whereCompsite.append("product_category",  mProductCategoryLst, "@>");
        whereCompsite.append("product_category",  "{*}");
        whereCompsite.setLogic(TSqlWhereCompsiteV2::Logic_Or);
        selector.whereRef().append(whereCompsite);
    }
    return selector;
}

// 导航栏的重置功能开始
void SysUserMgtV2::onNaviReset()
{
    mNaviView->clearValues();
    refresh();
}
// 导航栏的重置功能结束

// 返回一条where语句
QString SysUserMgtV2::uiloaderWhere()
{
    // 如果导航栏什么都没有输入，那就返回
    if (mNaviView == nullptr) {
        return "";
    }
    // filterDataMap里保存导航栏的所有数据
    auto filterDataMap = mNaviView->getAllValues().toVariant().toMap();
    QString searchSql;
    // 把filterDataMap放入迭代器iter里
    QMapIterator<QString,QVariant> iter(filterDataMap);
    // 循环判断迭代器里的数据
    while (iter.hasNext()) {
        iter.next();
        QString key = iter.key();
        QString value = iter.value().toString();
        // 如果值为空，继续
        if (value.isEmpty())
        {
            continue;
        }
        // 根据键来获取导航栏中的对象
        auto obj = mNaviView->getObject(key);
        if (obj !=nullptr) {
            QVariantMap userData = obj->property("tui_user_data").toMap();
            if (userData.contains("field_name")) {
                key = userData.value("field_name").toString();
            }
            QString operatorStr = "=";
            if (userData.contains("operator")) {
                operatorStr = userData.value("operator").toString();
            }
            if (operatorStr.compare("like", Qt::CaseInsensitive) == 0 ||
                    operatorStr.compare("ilike", Qt::CaseInsensitive) == 0) {
                value = QString("'%%1%'").arg(value);
            } else if (operatorStr.compare("in", Qt::CaseInsensitive) == 0) {
                value = QString("(%1)").arg(value);
            } else {
                if (iter.value().type() == QVariant::String) {
                    value = QString("'%1'").arg(value);
                }
            }
            QString searchStr = QString("%1 %2 %3").arg(key).arg(operatorStr).arg(value);
            if (!searchSql.isEmpty()) {
                searchSql.append(" and ");
            }
            searchSql.append(searchStr);
        }
    }
    return searchSql;
}

// 切换的槽开始
void SysUserMgtV2::onSelectionChanged()
{
    if (mTableView != nullptr) {
        // 选择多行数据
        mSelectedLst = mTableView->selectedPrimaryKeys();
        qDebug()<<mSelectedLst;
    int curIdInt = 0;
    if (!mSelectedLst.isEmpty()) {
        // 取出第一行数据
        curIdInt = mSelectedLst.value(0).toInt();
    }

    // 如果详情页的id与中间表格的id不相等，那就把现在的id给设置为详情页最近的(上一次）id
    if (mDetailView->uid().toInt() != curIdInt) {
        mDetailView->setLastUid(curIdInt);
        // 如果详情页的数据不再改变，那就把现在的id设为详情页的id
        if (!mDetailView->isDataModified()) {
            qDebug()<<"1111"<<curIdInt;
            mDetailView->setUid(curIdInt);
        }
    }
        refreshActionState();
    }
}
// 切换的槽结束

// 选择一条信息开始
QVariantList SysUserMgtV2::selectedItems()
{
    return mSelectedLst;
}
// 选择一条信息结束

void SysUserMgtV2::onDetailSaved(const QVariant &iUidStr)
{
    this->refresh(false);
    mTableView->selectRow(iUidStr);
}
