﻿#include "sysuserthreadv2.h"
#include <topcore/topclassabs.h>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tdataresponse.h>
#include <tdatabaseutil/tsqlconnectionpoolv2.h>
#include <tdatabaseutil/tsqlqueryv2.h>
#include <tdatabaseutil/tsqlselectorv2.h>

SysUserThreadV2::SysUserThreadV2(QObject *iParent)
    : TopClassThreadAbs(iParent)
{

}
SysUserThreadV2::~SysUserThreadV2()
{

}

void SysUserThreadV2::run()
{
    if (invokeName() == "LOAD_DATA") {
       loadData(invokeParameter().toInt());
    } else if (invokeName() == "SAVE_DATA") {
        saveData(invokeParameter().toMap());
    }
}

void SysUserThreadV2::loadData(int iIdInt)
{
    TDataResponse dataRes;
    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        TSqlSelectorV2 sqlSelector;
        sqlSelector.setTable("sys_user AS U LEFT JOIN pub_contacts AS C ON U.contcat_id = C.id");
        sqlSelector.setField("U.*,C.mail");
        sqlSelector.setWhere("U.id",QVariant(iIdInt));
        sqlSelector.setFieldFormat("product_category","array");
        sqlSelector.setFieldFormat("attr_data","json");
        QVariantMap dataMap = sqlQuery.selectMap(sqlSelector);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }
        sqlQuery.commit();
        dataRes.setData(dataMap);
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}

void SysUserThreadV2::saveData(const QVariantMap &iDataMap)
{
    // iDataMap就是用户信息栏的那些信息，由键值对组成，键是id,username之类的，值是用户输入的信息
    TDataResponse dataRes;
    TSqlQueryV2 sqlQuery(T_SQLCNT_POOL->getSqlDatabase());
    sqlQuery.begin();
    try {
        TSqlSelectorV2 sqlSelector;
        // 设置表名为sys_user
        sqlSelector.setTable("sys_user");
        // 清空当前字段列表，并追加字段名
        sqlSelector.setField("COUNT(*)");
        // 添加条件
        sqlSelector.addWhere("username", iDataMap.value("username"));
        sqlSelector.addWhere("id",iDataMap.value("id"),"!=");
        int count = sqlQuery.selectCount(sqlSelector);
        if (count > 0) {
            throw TError(ttr("'%1' already exists").arg(iDataMap.value("username").toString()),"ERROR","ALREADY_EXISTS");
        }

        // 更新pub_contacts表的mail
        QVariantMap contactMap;
        contactMap.insert("mail",iDataMap.value("mail"));
        QString username = iDataMap.value("username").toString();
        QString fullname = iDataMap.value("fullname").toString();
        QString name;
        if (!fullname.isEmpty() && (username != fullname)) {
            name = QString("%1(%2)").arg(fullname).arg(username);
        } else {
            name = username;
        }
        contactMap.insert("name",name);
        contactMap.insert("id", iDataMap.value("contcat_id").toInt());
        contactMap.insert("source","company");
        QString status = iDataMap.value("status").toString();
        if (iDataMap.value("status").toString() == "active")
        {
            status = "active";
        } else {
            status = "inactive";
        }
        contactMap.insert("status",status);
        // 从这儿开始不懂
        TopClassAbs *parentModule = static_cast<TopClassAbs *>(this->parent());
        if (parentModule) {
            if (parentModule->isHookExists("handleContactData")) {
                contactMap = parentModule->callHooksQuick("handleCountactData", QVariantList()<<contactMap<<iDataMap).toVariant().toMap();
            }
        }
        QStringList contactFields = contactMap.keys();
        contactFields.removeOne("id");
        // TSqlInserterV2类用于增加数据，sqlInserter可以理解为插入器
        TSqlInserterV2 sqlInserter;
        // 设置表为"pub_contacts",返回自身引用
        sqlInserter.setTable("pub_contacts");
        // setData()设置值为contactMap
        sqlInserter.setData(contactMap);
        // setField()设置字段列表，返回自身引用
        sqlInserter.setField(contactFields);
        // 设置id为唯一性字段列表，唯一性字段就是不可以有重复值的字段
        sqlInserter.setUniqueField("id");
        // 设置自增加字段为id，返回自身引用，自增字段就是说不用自己输入值，它会自己自己在前面数值的基础上增长
        sqlInserter.setAutoIncrementField("id");
        // 插入时，返回最后插入的id
        QVariant contactId = sqlQuery.replaceRow(sqlInserter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

//         TSqlInserterV2 sqlInserter;
//         sqlInserter.setTable("pub_contacts");
//         QString username = iDataMap.value("username").toString();
//         QString fullname = iDataMap.value("fullname").toString();
//         QString name;
//         if (!fullname.isEmpty() && (username != fullname)) {
//              name = QString("%1(%2)").arg(fullname).arg(username);
//           } else {
//              name = username;
//           }
//           QString status = iDataMap.value("status").toString();
//           if (iDataMap.value("status").toString() == "active")
//           {
//              status = "active";
//           } else {
//              status = "inactive";
//           }

//           sqlInserter.setField(QStringList()<<"mail"<<"name"<<"source"<<"status");

//           sqlInserter.insert("id", iDataMap.value("contcat_id").toInt());
//           sqlInserter.insert("name",name);
//           sqlInserter.insert("mail",iDataMap.value("mail"));
//           sqlInserter.insert("source","company");
//           sqlInserter.insert("status",status);
//           sqlInserter.setUniqueField("id");
//           sqlInserter.setAutoIncrementField("id");

//           //setUniqueField("id"):repalce()会知道检查setUniqueField()设置的参数
//            //replaceRow():查数据库里id是否已经有了，如果没有，就创建id
//           QVariant contactId = sqlQuery.replaceRow(sqlInserter);
//           if (sqlQuery.lastError().isValid()) {
//               throw sqlQuery.lastError();
//        }


        // 更新sys_commu_grp_detail
        // TSqlUpdaterV2用于改数据
        TSqlUpdaterV2 sqlUpdater;
        sqlUpdater.setTable("sys_commu_grp_detail");
        sqlUpdater.setField(QStringList()<<"address"<<"name");
        QVariantMap commuGrpDetailDataMap;
        commuGrpDetailDataMap.insert("address",iDataMap.value("mail"));
        commuGrpDetailDataMap.insert("name",name);
        sqlUpdater.setData(commuGrpDetailDataMap);
        // sqlUpdater.insert("address",iDataMap.value("mail"));
        // sqlUpdater.insert("name",name);
        TSqlWhereCompsiteV2 commuGrpDetailWhere;
        // 追加where条件：src的值是pub_contacts，返回自身引用
        commuGrpDetailWhere.append("src_type","pub_contacts");
        commuGrpDetailWhere.append("src_uid",iDataMap.value("contcat_id").toString());
        // 设置逻辑关系，Logic_And=0；逻辑与=0
        commuGrpDetailWhere.setLogic(TSqlWhereCompsiteV2::Logic_And);
        sqlUpdater.setWhere(commuGrpDetailWhere);
        sqlQuery.updateRow(sqlUpdater);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        // 更新sys_user表
        // dataMap就是用户输入的信息
        QVariantMap dataMap = iDataMap;
        QStringList fieldLst = dataMap.keys();
        dataMap.insert("contcat_id",contactId);

        QStringList productCategory = dataMap.value("product_category").toStringList();
        if (productCategory.contains("*")) {
            productCategory = QStringList() << "*";
        }
        dataMap.insert("product_category", productCategory);
        if (iDataMap.value("id").toInt() == 0) {
            // 新建时插入默认密码
            QString password = iDataMap.value("password").toString();
            if (password.isEmpty()) {
                password = QStringLiteral("123456");
            }
            dataMap.insert("password",TDataParse::getVariantMd5(password));
        } else {
            fieldLst.removeOne("password");
        }
        fieldLst.removeOne("id");
        fieldLst.removeOne("mail");
        // 这句不懂是什么意思
        qDebug().noquote().nospace()<<TDataParse::variant2JsonStr(dataMap);
        // 清除之前执行的语句
        sqlInserter.clear();
        sqlInserter.setTable("sys_user");
        sqlInserter.setData(dataMap);
        sqlInserter.setField(fieldLst);
        sqlInserter.setUpdatePolicy("attr_data", "json_merge");
        sqlInserter.setUniqueField("id");
        sqlInserter.setAutoIncrementField("id");
        QVariant userId = sqlQuery.replaceRow(sqlInserter);
        if (sqlQuery.lastError().isValid()) {
            throw sqlQuery.lastError();
        }

        sqlQuery.commit();
        dataRes.setData(userId);
        setInvokeResult(dataRes.toVariantMap());
        return;
    } catch (const TError &err) {
        dataRes.setError(err);
        // 任何错误都会引起程序结束执行
    } catch (...) {
        dataRes.setErrText(ttr("Unknow Error!"));
    }
    sqlQuery.rollback();
    setInvokeResult(dataRes.toVariantMap());
}
