﻿#include "sysuserv2.h"
#include <QGraphicsDropShadowEffect>
#include <QResizeEvent>
#include <QToolBar>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <tbaseutil/tdataresponse.h>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/ttheme.h>
#include <tbaseutil/tenumlist.h>
#include <tdatabaseutil/tsqlselectorv2.h>
#include <topcore/topcore.h>
#include <topcore/topclasssqlthread.h>
#include <topcore/topenummanager.h>
#include <toputil/t.h>
#include <twidget/tmessagebar.h>
#include <twidget/ttableview.h>
#include <twidget/ttableviewdialog.h>
#include <twidget/tuiloader.h>
#include "sysuserthreadv2.h"

SysUserV2::SysUserV2(const QString &iModuleNameStr, const QVariantMap &iUrlPars, QWidget *iParent)
    : TopClassAbs(iParent)
{
    this->setLicenseKey("sys_common");
    this->initModule(iModuleNameStr, iUrlPars);

    QVariant prodCfg = iUrlPars.value("product_category");
    if (prodCfg.type() == QVariant::List || prodCfg.type() == QVariant::StringList) {
        for (QVariant row : prodCfg.toList()) {
            mProductCategoryLst.append(row.toString());
        }
    } else if (!prodCfg.toString().isEmpty()) {
        mProductCategoryLst.append(prodCfg.toString());
    }

    if (mProductCategoryLst.isEmpty()) {
        mProductCategoryLst.append(APP->productCategory());
    }

    // 设置窗体布局
    QWidget *centerWidget = new QWidget(this);
    this->setCentralWidget(centerWidget);
    // 总体垂直布局,也就是说centerWidget是垂直布局
    QVBoxLayout *vboxlayout = new QVBoxLayout(centerWidget);
    vboxlayout->setMargin(0);
    vboxlayout->setSpacing(0);
    // 第二行水平布局
    hboxlayout = new QHBoxLayout();
    vboxlayout->addLayout(hboxlayout, 1);
    // 设置第二行的样式
    hboxlayout->setMargin(TTHEME_DP(16));
    hboxlayout->setSpacing(0);
    // 给第一行里面添加保存、取消、复制的按钮控件
    if (QToolBar *toolbar = qobject_cast<QToolBar *>(uim()->getWidget("MAIN_TOOLBAR"))) {
        vboxlayout->insertWidget(0, toolbar, 0);
    }
    // 给第二行里添加UI界面
    // 首先创建一个UI界面
    mUiLoader = new TUiLoader(this);
    // 然后把UI加入水平布局中
    hboxlayout->addWidget(mUiLoader,1);
    // 设置UI界面的样式
    mUiLoader->setProperty("SS_BG", "PANEL");
    mUiLoader->setProperty("SS_BORDER", 1);
    mUiLoader->setMaximumWidth(TTHEME_DP(config("maximum_size.width",800).toInt()));
    // 此时UI界面还是空的，给UI界面里面添加内容
    mUiLoader->setScriptEngine(APP->scriptEngine());
    mUiLoader->setSelf(this);
    mUiLoader->setUiStr(ui("user-info").toString());
    // 当UI界面的数据被更改时，......
    connect(mUiLoader, SIGNAL(dataChanged()),this,SLOT(setDataModified()));

    // 恢复窗体尺寸及布局
    restoreSizeState();

    uidChangeEvent(this->uid());
    // 刷新Action的状态
    refreshActionState();
}

// 下次打开的页面保持上次页面的尺寸
SysUserV2::~SysUserV2(){
    saveSizeState();
}

// 设置用户信息界面的页面功能开始
// 此时页面上的数据还只在用户信息界面
void SysUserV2::create()
{
    // 显示上次的界面
    setLastUid(this->uid());
    // 设置为id==0的用户，此用户默认会清空界面
    setUid(0,true);
    //
    QVariantMap defaultValueMap;
    defaultValueMap.insert("product_category",mProductCategoryLst);
    defaultValueMap = defaultValueMap.unite(config("default_value").toMap());
    mUiLoader->setState("status", "disable");
    setData(defaultValueMap);
    setDataModified();
}
// 设置用户信息界面的页面功能结束

void SysUserV2::setData(const QVariantMap &iDataMap)
{
    mUiLoader->loadValues(iDataMap, false);
    if (iDataMap.contains("username")) {
        setTitle(ttr("User-%1").arg(iDataMap.value("username").toString()));
    }
}

// 获取UI界面的数据开始
QVariantMap SysUserV2::getData() const
{
    // 首先获取当前UI界面所有的数据
    QVariantMap dataMap = mUiLoader->getAllValues(true).toVariant().toMap();
    /* 因为.ui.js里面有些name里有.比如attr_data.password_validity_control，然后呢用户输入一个值，比如6，那么现在这个键值对就是attr_data.password_validity_control：6
     * .的左边是attr_data，右边是validity_period或者password_validity_control
     * 数据库里的一个字段名分别对应了一个键，所以为了后续把数据存入数据库做准备，
     * 需要先把数据以.为分割点分开
     */
    QStringList keys = dataMap.keys();
    for (QString key: keys) {
        if (key.contains(".")) {
            // 在.两边，first()渠道的是左边的字符串，last()取到的是右边的字符串
            // 把.左边的字符串给KeyL
            QString keyL = key.split(".").first();
            // 把右边的字符串给keyR
            QString keyR = key.split(".").last();
            // 用map来存储将原数据更改键值后的数据
            QVariantMap map;
            map.insert(keyR, dataMap.value(key));
            dataMap.insert(keyL, map);
            dataMap.remove(key);
        }
    }
    if (this->uid().toInt() == 0) {
        QVariantMap attrData = dataMap.value("attr_data").toMap();
        attrData.insert("effective_date", APP->getServerToday());
        dataMap.insert("attr_data", attrData);
    }
    if (dataMap.value("product_category").toStringList().isEmpty()) {
        dataMap.insert("product_category", mProductCategoryLst);
    }

    dataMap.insert("id", this->uid());
    return dataMap;
}
// 获取UI界面的数据结束

// 保存功能开始
// 保存数据的关键在于，将获取的数据放入数据库，再取出来放入中间表格界面
// 将获取的数据放入数据库->已经由doThreadWork实现
void SysUserV2::saveData()
{
    //在用户提交时、保存前，判断用户详情栏输入的信息的格式是否有效，并将错误信息返回给errLst
    QVariantList errLst = mUiLoader->validateAll("COMMIT", true, "ERROR");
    // 如果有错,那就遍历错误，并把错误信息给添加到errStrLst,然后提醒保存错误失败，并换行提醒错误信息，退出保存功能
    if (!errLst.isEmpty()) {
        QStringList errStrLst;
        for (QVariant err : errLst) {
            errStrLst.append(err.toMap().value("text").toString());
        }
        alertError(ttr("Saving data failed!"), errStrLst.join("\n"));
        return  ;
    }
    // 如果没错，就执行下列操作
    // 1.存储当前界面信息->删除这句话，竖线会变成横线
    t::saving(this);
    //这两条就是关键：获取数据后，将获取的正确的数据存入数据库
    QVariantMap saveData = getData();
    QVariant data = doThreadWork(new SysUserThreadV2(this), "SAVE_DATA", saveData);
    unloading();
    TDataResponse dataRes(data.toMap());
    if (dataRes.hasError()) {
        alertError(ttr("Save data failed!"), dataRes.errText());
    } else {
        setUid(dataRes.data().toInt());
        emit dataSaved(this->uid());
        APP->notify("sys-user-save");
        alertOk(ttr("Data saved"));
    }
}
// 保存功能结束

// 重载功能开始 -设置用户ID
void SysUserV2::reload()
{
    QString uidStr;
    if (lastUid().isEmpty()) {
        uidStr = uid();
    } else {
        uidStr = lastUid();
    }
    setUid(uidStr,true);
}
// 重载功能结束

void SysUserV2::clearData()
{
    //在用户详情界面上，载入数据，因为传入的参数是空的，所以将详情页面置空
    mUiLoader->loadValues(QVariantMap(),true);
}

// 这是setUid()触发的事件
void SysUserV2::uidChangeEvent(const QString &iUidStr)
{
    // 如果是新用户，就把用户详情页的数据置空
    if (iUidStr.toInt() == 0) {
        setTitle(ttr("New User"));
        clearData();
    } else {
        t::loading(this);
        // 把载入的数据存储到data里
        QVariant data = doThreadWork(new SysUserThreadV2(this),"LOAD_DATA",QVariant(this->uid()));
        unloading();
        TDataResponse dataRes(data.toMap());
        if (dataRes.hasError()) {
            alertError(ttr("Load data failed"), dataRes.errText());
        } else {
            clearData();
            QVariantMap dataMap = dataRes.data().toMap();
            QVariantMap attrData = dataMap.value("attr_data").toMap();
            for (QString key: attrData.keys()) {
                dataMap.insert("attr_data." + key, attrData.value(key));
            }
            setData(dataMap);
            alertOk(ttr("Data loaded"));
        }
    }
    mUiLoader->setState("status","enable");
    setDataModified(false);
}


